[][src]Type Definition sled_extensions::structured::expiring::Tree

type Tree<V, E> = Tree<V, E, E>;

A basic expiring strutured tree

This tree is an alias for a generic expiring tree, but asserts that the metadata encoding is the same as the value encoding.